@@ -7,7 +7,6 @@
 
 from __future__ import absolute_import
 
-import hashlib
 import io
 import os
 import threading
@@ -25,7 +24,10 @@ from mercurial import (
     util,
     wireprotov1peer,
 )
-from mercurial.utils import procutil
+from mercurial.utils import (
+    hashutil,
+    procutil,
+)
 
 from . import (
     constants,
@@ -43,52 +45,61 @@ fetchmisses = 0
 
 _lfsmod = None
 
+
 def getcachekey(reponame, file, id):
-    pathhash = node.hex(hashlib.sha1(file).digest())
+    pathhash = node.hex(hashutil.sha1(file).digest())
     return os.path.join(reponame, pathhash[:2], pathhash[2:], id)
 
+
 def getlocalkey(file, id):
-    pathhash = node.hex(hashlib.sha1(file).digest())
+    pathhash = node.hex(hashutil.sha1(file).digest())
     return os.path.join(pathhash, id)
 
-def peersetup(ui, peer):
 
+def peersetup(ui, peer):
     class remotefilepeer(peer.__class__):
         @wireprotov1peer.batchable
         def x_rfl_getfile(self, file, node):
-            if not self.capable('x_rfl_getfile'):
+            if not self.capable(b'x_rfl_getfile'):
                 raise error.Abort(
-                    'configured remotefile server does not support getfile')
+                    b'configured remotefile server does not support getfile'
+                )
             f = wireprotov1peer.future()
-            yield {'file': file, 'node': node}, f
-            code, data = f.value.split('\0', 1)
+            yield {b'file': file, b'node': node}, f
+            code, data = f.value.split(b'\0', 1)
             if int(code):
                 raise error.LookupError(file, node, data)
             yield data
 
         @wireprotov1peer.batchable
         def x_rfl_getflogheads(self, path):
-            if not self.capable('x_rfl_getflogheads'):
-                raise error.Abort('configured remotefile server does not '
-                                  'support getflogheads')
+            if not self.capable(b'x_rfl_getflogheads'):
+                raise error.Abort(
+                    b'configured remotefile server does not '
+                    b'support getflogheads'
+                )
             f = wireprotov1peer.future()
-            yield {'path': path}, f
-            heads = f.value.split('\n') if f.value else []
+            yield {b'path': path}, f
+            heads = f.value.split(b'\n') if f.value else []
             yield heads
 
         def _updatecallstreamopts(self, command, opts):
-            if command != 'getbundle':
+            if command != b'getbundle':
                 return
-            if (constants.NETWORK_CAP_LEGACY_SSH_GETFILES
-                not in self.capabilities()):
+            if (
+                constants.NETWORK_CAP_LEGACY_SSH_GETFILES
+                not in self.capabilities()
+            ):
                 return
             if not util.safehasattr(self, '_localrepo'):
                 return
-            if (constants.SHALLOWREPO_REQUIREMENT
-                not in self._localrepo.requirements):
+            if (
+                constants.SHALLOWREPO_REQUIREMENT
+                not in self._localrepo.requirements
+            ):
                 return
 
-            bundlecaps = opts.get('bundlecaps')
+            bundlecaps = opts.get(b'bundlecaps')
             if bundlecaps:
                 bundlecaps = [bundlecaps]
             else:
@@ -103,19 +114,20 @@ def peersetup(ui, peer):
             # do this more cleanly.
             bundlecaps.append(constants.BUNDLE2_CAPABLITY)
             if self._localrepo.includepattern:
-                patterns = '\0'.join(self._localrepo.includepattern)
-                includecap = "includepattern=" + patterns
+                patterns = b'\0'.join(self._localrepo.includepattern)
+                includecap = b"includepattern=" + patterns
                 bundlecaps.append(includecap)
             if self._localrepo.excludepattern:
-                patterns = '\0'.join(self._localrepo.excludepattern)
-                excludecap = "excludepattern=" + patterns
+                patterns = b'\0'.join(self._localrepo.excludepattern)
+                excludecap = b"excludepattern=" + patterns
                 bundlecaps.append(excludecap)
-            opts['bundlecaps'] = ','.join(bundlecaps)
+            opts[b'bundlecaps'] = b','.join(bundlecaps)
 
         def _sendrequest(self, command, args, **opts):
             self._updatecallstreamopts(command, args)
-            return super(remotefilepeer, self)._sendrequest(command, args,
-                                                            **opts)
+            return super(remotefilepeer, self)._sendrequest(
+                command, args, **opts
+            )
 
         def _callstream(self, command, **opts):
             supertype = super(remotefilepeer, self)
@@ -125,11 +137,13 @@ def peersetup(ui, peer):
 
     peer.__class__ = remotefilepeer
 
+
 class cacheconnection(object):
     """The connection for communicating with the remote cache. Performs
     gets and sets by communicating with an external process that has the
     cache-specific implementation.
     """
+
     def __init__(self):
         self.pipeo = self.pipei = self.pipee = None
         self.subprocess = None
@@ -137,9 +151,10 @@ class cacheconnection(object):
 
     def connect(self, cachecommand):
         if self.pipeo:
-            raise error.Abort(_("cache connection already open"))
-        self.pipei, self.pipeo, self.pipee, self.subprocess = (
-            procutil.popen4(cachecommand))
+            raise error.Abort(_(b"cache connection already open"))
+        self.pipei, self.pipeo, self.pipee, self.subprocess = procutil.popen4(
+            cachecommand
+        )
         self.connected = True
 
     def close(self):
@@ -148,9 +163,10 @@ class cacheconnection(object):
                 pipe.close()
             except Exception:
                 pass
+
         if self.connected:
             try:
-                self.pipei.write("exit\n")
+                self.pipei.write(b"exit\n")
             except Exception:
                 pass
             tryclose(self.pipei)
@@ -190,8 +206,10 @@ class cacheconnection(object):
 
         return result
 
+
 def _getfilesbatch(
-        remote, receivemissing, progresstick, missed, idmap, batchsize):
+    remote, receivemissing, progresstick, missed, idmap, batchsize
+):
     # Over http(s), iterbatch is a streamy method and we can start
     # looking at results early. This means we send one (potentially
     # large) request, but then we show nice progress as we process
@@ -205,22 +223,25 @@ def _getfilesbatch(
     with remote.commandexecutor() as e:
         futures = []
         for m in missed:
-            futures.append(e.callcommand('x_rfl_getfile', {
-                'file': idmap[m],
-                'node': m[-40:]
-            }))
+            futures.append(
+                e.callcommand(
+                    b'x_rfl_getfile', {b'file': idmap[m], b'node': m[-40:]}
+                )
+            )
 
         for i, m in enumerate(missed):
             r = futures[i].result()
             futures[i] = None  # release memory
             file_ = idmap[m]
             node = m[-40:]
-            receivemissing(io.BytesIO('%d\n%s' % (len(r), r)), file_, node)
+            receivemissing(io.BytesIO(b'%d\n%s' % (len(r), r)), file_, node)
             progresstick()
 
+
 def _getfiles_optimistic(
-    remote, receivemissing, progresstick, missed, idmap, step):
-    remote._callstream("x_rfl_getfiles")
+    remote, receivemissing, progresstick, missed, idmap, step
+):
+    remote._callstream(b"x_rfl_getfiles")
     i = 0
     pipeo = remote._pipeo
     pipei = remote._pipei
@@ -233,7 +254,7 @@ def _getfiles_optimistic(
             # issue new request
             versionid = missingid[-40:]
             file = idmap[missingid]
-            sshrequest = "%s%s\n" % (versionid, file)
+            sshrequest = b"%s%s\n" % (versionid, file)
             pipeo.write(sshrequest)
         pipeo.flush()
 
@@ -245,12 +266,14 @@ def _getfiles_optimistic(
             progresstick()
 
     # End the command
-    pipeo.write('\n')
+    pipeo.write(b'\n')
     pipeo.flush()
 
+
 def _getfiles_threaded(
-    remote, receivemissing, progresstick, missed, idmap, step):
-    remote._callstream("getfiles")
+    remote, receivemissing, progresstick, missed, idmap, step
+):
+    remote._callstream(b"getfiles")
     pipeo = remote._pipeo
     pipei = remote._pipei
 
@@ -258,9 +281,10 @@ def _getfiles_threaded(
         for missingid in missed:
             versionid = missingid[-40:]
             file = idmap[missingid]
-            sshrequest = "%s%s\n" % (versionid, file)
+            sshrequest = b"%s%s\n" % (versionid, file)
             pipeo.write(sshrequest)
         pipeo.flush()
+
     writerthread = threading.Thread(target=writer)
     writerthread.daemon = True
     writerthread.start()
@@ -273,26 +297,29 @@ def _getfiles_threaded(
 
     writerthread.join()
     # End the command
-    pipeo.write('\n')
+    pipeo.write(b'\n')
     pipeo.flush()
 
+
 class fileserverclient(object):
     """A client for requesting files from the remote file server.
     """
+
     def __init__(self, repo):
         ui = repo.ui
         self.repo = repo
         self.ui = ui
-        self.cacheprocess = ui.config("remotefilelog", "cacheprocess")
+        self.cacheprocess = ui.config(b"remotefilelog", b"cacheprocess")
         if self.cacheprocess:
             self.cacheprocess = util.expandpath(self.cacheprocess)
 
         # This option causes remotefilelog to pass the full file path to the
         # cacheprocess instead of a hashed key.
         self.cacheprocesspasspath = ui.configbool(
-            "remotefilelog", "cacheprocess.includepath")
+            b"remotefilelog", b"cacheprocess.includepath"
+        )
 
-        self.debugoutput = ui.configbool("remotefilelog", "debug")
+        self.debugoutput = ui.configbool(b"remotefilelog", b"debug")
 
         self.remotecache = cacheconnection()
 
@@ -318,19 +345,19 @@ class fileserverclient(object):
 
         repo = self.repo
         total = len(fileids)
-        request = "get\n%d\n" % total
+        request = b"get\n%d\n" % total
         idmap = {}
         reponame = repo.name
         for file, id in fileids:
             fullid = getcachekey(reponame, file, id)
             if self.cacheprocesspasspath:
-                request += file + '\0'
-            request += fullid + "\n"
+                request += file + b'\0'
+            request += fullid + b"\n"
             idmap[fullid] = file
 
         cache.request(request)
 
-        progress = self.ui.makeprogress(_('downloading'), total=total)
+        progress = self.ui.makeprogress(_(b'downloading'), total=total)
         progress.update(0)
 
         missed = []
@@ -341,14 +368,18 @@ class fileserverclient(object):
                 for missingid in idmap:
                     if not missingid in missedset:
                         missed.append(missingid)
-                self.ui.warn(_("warning: cache connection closed early - " +
-                    "falling back to server\n"))
+                self.ui.warn(
+                    _(
+                        b"warning: cache connection closed early - "
+                        + b"falling back to server\n"
+                    )
+                )
                 break
-            if missingid == "0":
+            if missingid == b"0":
                 break
-            if missingid.startswith("_hits_"):
+            if missingid.startswith(b"_hits_"):
                 # receive progress reports
-                parts = missingid.split("_")
+                parts = missingid.split(b"_")
                 progress.increment(int(parts[2]))
                 continue
 
@@ -359,8 +390,14 @@ class fileserverclient(object):
 
         fromcache = total - len(missed)
         progress.update(fromcache, total=total)
-        self.ui.log("remotefilelog", "remote cache hit rate is %r of %r\n",
-                    fromcache, total, hit=fromcache, total=total)
+        self.ui.log(
+            b"remotefilelog",
+            b"remote cache hit rate is %r of %r\n",
+            fromcache,
+            total,
+            hit=fromcache,
+            total=total,
+        )
 
         oldumask = os.umask(0o002)
         try:
@@ -375,51 +412,74 @@ class fileserverclient(object):
                     with self._connect() as conn:
                         remote = conn.peer
                         if remote.capable(
-                                constants.NETWORK_CAP_LEGACY_SSH_GETFILES):
+                            constants.NETWORK_CAP_LEGACY_SSH_GETFILES
+                        ):
                             if not isinstance(remote, _sshv1peer):
-                                raise error.Abort('remotefilelog requires ssh '
-                                                  'servers')
-                            step = self.ui.configint('remotefilelog',
-                                                     'getfilesstep')
-                            getfilestype = self.ui.config('remotefilelog',
-                                                          'getfilestype')
-                            if getfilestype == 'threaded':
+                                raise error.Abort(
+                                    b'remotefilelog requires ssh servers'
+                                )
+                            step = self.ui.configint(
+                                b'remotefilelog', b'getfilesstep'
+                            )
+                            getfilestype = self.ui.config(
+                                b'remotefilelog', b'getfilestype'
+                            )
+                            if getfilestype == b'threaded':
                                 _getfiles = _getfiles_threaded
                             else:
                                 _getfiles = _getfiles_optimistic
-                            _getfiles(remote, self.receivemissing,
-                                      progress.increment, missed, idmap, step)
-                        elif remote.capable("x_rfl_getfile"):
-                            if remote.capable('batch'):
+                            _getfiles(
+                                remote,
+                                self.receivemissing,
+                                progress.increment,
+                                missed,
+                                idmap,
+                                step,
+                            )
+                        elif remote.capable(b"x_rfl_getfile"):
+                            if remote.capable(b'batch'):
                                 batchdefault = 100
                             else:
                                 batchdefault = 10
                             batchsize = self.ui.configint(
-                                'remotefilelog', 'batchsize', batchdefault)
+                                b'remotefilelog', b'batchsize', batchdefault
+                            )
                             self.ui.debug(
                                 b'requesting %d files from '
-                                b'remotefilelog server...\n' % len(missed))
+                                b'remotefilelog server...\n' % len(missed)
+                            )
                             _getfilesbatch(
-                                remote, self.receivemissing, progress.increment,
-                                missed, idmap, batchsize)
+                                remote,
+                                self.receivemissing,
+                                progress.increment,
+                                missed,
+                                idmap,
+                                batchsize,
+                            )
                         else:
-                            raise error.Abort("configured remotefilelog server"
-                                             " does not support remotefilelog")
+                            raise error.Abort(
+                                b"configured remotefilelog server"
+                                b" does not support remotefilelog"
+                            )
 
-                    self.ui.log("remotefilefetchlog",
-                                "Success\n",
-                                fetched_files = progress.pos - fromcache,
-                                total_to_fetch = total - fromcache)
+                    self.ui.log(
+                        b"remotefilefetchlog",
+                        b"Success\n",
+                        fetched_files=progress.pos - fromcache,
+                        total_to_fetch=total - fromcache,
+                    )
                 except Exception:
-                    self.ui.log("remotefilefetchlog",
-                                "Fail\n",
-                                fetched_files = progress.pos - fromcache,
-                                total_to_fetch = total - fromcache)
+                    self.ui.log(
+                        b"remotefilefetchlog",
+                        b"Fail\n",
+                        fetched_files=progress.pos - fromcache,
+                        total_to_fetch=total - fromcache,
+                    )
                     raise
                 finally:
                     self.ui.verbose = verbose
                 # send to memcache
-                request = "set\n%d\n%s\n" % (len(missed), "\n".join(missed))
+                request = b"set\n%d\n%s\n" % (len(missed), b"\n".join(missed))
                 cache.request(request)
 
             progress.complete()
@@ -432,21 +492,25 @@ class fileserverclient(object):
     def receivemissing(self, pipe, filename, node):
         line = pipe.readline()[:-1]
         if not line:
-            raise error.ResponseError(_("error downloading file contents:"),
-                                      _("connection closed early"))
+            raise error.ResponseError(
+                _(b"error downloading file contents:"),
+                _(b"connection closed early"),
+            )
         size = int(line)
         data = pipe.read(size)
         if len(data) != size:
-            raise error.ResponseError(_("error downloading file contents:"),
-                                      _("only received %s of %s bytes")
-                                      % (len(data), size))
+            raise error.ResponseError(
+                _(b"error downloading file contents:"),
+                _(b"only received %s of %s bytes") % (len(data), size),
+            )
 
-        self.writedata.addremotefilelognode(filename, bin(node),
-                                             zlib.decompress(data))
+        self.writedata.addremotefilelognode(
+            filename, bin(node), zlib.decompress(data)
+        )
 
     def connect(self):
         if self.cacheprocess:
-            cmd = "%s %s" % (self.cacheprocess, self.writedata._path)
+            cmd = b"%s %s" % (self.cacheprocess, self.writedata._path)
             self.remotecache.connect(cmd)
         else:
             # If no cache process is specified, we fake one that always
@@ -462,11 +526,11 @@ class fileserverclient(object):
                     pass
 
                 def request(self, value, flush=True):
-                    lines = value.split("\n")
-                    if lines[0] != "get":
+                    lines = value.split(b"\n")
+                    if lines[0] != b"get":
                         return
                     self.missingids = lines[2:-1]
-                    self.missingids.append('0')
+                    self.missingids.append(b'0')
 
                 def receiveline(self):
                     if len(self.missingids) > 0:
@@ -477,26 +541,33 @@ class fileserverclient(object):
 
     def close(self):
         if fetches:
-            msg = ("%d files fetched over %d fetches - " +
-                   "(%d misses, %0.2f%% hit ratio) over %0.2fs\n") % (
-                       fetched,
-                       fetches,
-                       fetchmisses,
-                       float(fetched - fetchmisses) / float(fetched) * 100.0,
-                       fetchcost)
+            msg = (
+                b"%d files fetched over %d fetches - "
+                + b"(%d misses, %0.2f%% hit ratio) over %0.2fs\n"
+            ) % (
+                fetched,
+                fetches,
+                fetchmisses,
+                float(fetched - fetchmisses) / float(fetched) * 100.0,
+                fetchcost,
+            )
             if self.debugoutput:
                 self.ui.warn(msg)
-            self.ui.log("remotefilelog.prefetch", msg.replace("%", "%%"),
+            self.ui.log(
+                b"remotefilelog.prefetch",
+                msg.replace(b"%", b"%%"),
                 remotefilelogfetched=fetched,
                 remotefilelogfetches=fetches,
                 remotefilelogfetchmisses=fetchmisses,
-                remotefilelogfetchtime=fetchcost * 1000)
+                remotefilelogfetchtime=fetchcost * 1000,
+            )
 
         if self.remotecache.connected:
             self.remotecache.close()
 
-    def prefetch(self, fileids, force=False, fetchdata=True,
-                 fetchhistory=False):
+    def prefetch(
+        self, fileids, force=False, fetchdata=True, fetchhistory=False
+    ):
         """downloads the given file versions to the cache
         """
         repo = self.repo
@@ -506,8 +577,11 @@ class fileserverclient(object):
             # - we don't use .hgtags
             # - workingctx produces ids with length 42,
             #   which we skip since they aren't in any cache
-            if (file == '.hgtags' or len(id) == 42
-                or not repo.shallowmatch(file)):
+            if (
+                file == b'.hgtags'
+                or len(id) == 42
+                or not repo.shallowmatch(file)
+            ):
                 continue
 
             idstocheck.append((file, bin(id)))
@@ -517,7 +591,8 @@ class fileserverclient(object):
         if force:
             datastore = contentstore.unioncontentstore(*repo.shareddatastores)
             historystore = metadatastore.unionmetadatastore(
-                *repo.sharedhistorystores)
+                *repo.sharedhistorystores
+            )
 
         missingids = set()
         if fetchdata:
@@ -531,9 +606,12 @@ class fileserverclient(object):
         if nullids:
             missingids = [(f, id) for f, id in missingids if id != nullid]
             repo.ui.develwarn(
-                ('remotefilelog not fetching %d null revs'
-                 ' - this is likely hiding bugs' % nullids),
-                config='remotefilelog-ext')
+                (
+                    b'remotefilelog not fetching %d null revs'
+                    b' - this is likely hiding bugs' % nullids
+                ),
+                config=b'remotefilelog-ext',
+            )
         if missingids:
             global fetches, fetched, fetchcost
             fetches += 1
@@ -542,24 +620,27 @@ class fileserverclient(object):
             # let's log that information for debugging.
             if fetches >= 15 and fetches < 18:
                 if fetches == 15:
-                    fetchwarning = self.ui.config('remotefilelog',
-                                                  'fetchwarning')
+                    fetchwarning = self.ui.config(
+                        b'remotefilelog', b'fetchwarning'
+                    )
                     if fetchwarning:
-                        self.ui.warn(fetchwarning + '\n')
+                        self.ui.warn(fetchwarning + b'\n')
                 self.logstacktrace()
             missingids = [(file, hex(id)) for file, id in sorted(missingids)]
             fetched += len(missingids)
             start = time.time()
             missingids = self.request(missingids)
             if missingids:
-                raise error.Abort(_("unable to download %d files") %
-                                  len(missingids))
+                raise error.Abort(
+                    _(b"unable to download %d files") % len(missingids)
+                )
             fetchcost += time.time() - start
             self._lfsprefetch(fileids)
 
     def _lfsprefetch(self, fileids):
         if not _lfsmod or not util.safehasattr(
-                self.repo.svfs, 'lfslocalblobstore'):
+            self.repo.svfs, b'lfslocalblobstore'
+        ):
             return
         if not _lfsmod.wrapper.candownload(self.repo):
             return
@@ -569,7 +650,7 @@ class fileserverclient(object):
             node = bin(id)
             rlog = self.repo.file(file)
             if rlog.flags(node) & revlog.REVIDX_EXTSTORED:
-                text = rlog.revision(node, raw=True)
+                text = rlog.rawdata(node)
                 p = _lfsmod.pointer.deserialize(text)
                 oid = p.oid()
                 if not store.has(oid):
@@ -580,5 +661,9 @@ class fileserverclient(object):
 
     def logstacktrace(self):
         import traceback
-        self.ui.log('remotefilelog', 'excess remotefilelog fetching:\n%s\n',
-                    ''.join(traceback.format_stack()))
+
+        self.ui.log(
+            b'remotefilelog',
+            b'excess remotefilelog fetching:\n%s\n',
+            b''.join(pycompat.sysbytes(s) for s in traceback.format_stack()),
+        )
